class Stasis
  module Extensions
    class Assets < Stasis::Plugin
      before_all :before_all
      reset :reset
      action_method :assets

      def initialize(stasis)
        @stasis = stasis
        reset
      end

      def assets
        @assets
      end

      def before_all
        define_helpers
        fingerprint_assets
        Stasis::Options.set_template_option 'scss', { asset_resolver: self }
      end

      def reset
        @assets = {}
      end

      def resolve_path(path, dirpath)
        path = File.expand_path(path, dirpath).sub("#{@stasis.root}/", '')
        "/#{assets[path] || path}"
      end

      protected

      def define_helpers
        @stasis.controller.helpers do
          def asset_path(path)
            return path if path =~ %r"\A(?:https?:)?//[^/]+"
            return asset_path(path[1..-1]) if path =~ %r"\A/"
            "/#{assets[path] || path}"
          end

          def asset_url(path)
            "#{root_url}#{asset_path(path)}"
          end
        end
      end

      def fingerprint_asset(source)
        digest = Digest::MD5.file(source)
        render_asset source, source.sub(/\.(?:erb|haml|scss)$/, ''), source.sub(/(\.(?:\w+))(\.(?:erb|haml|scss))?$/) { |m| "-#{digest}#{$1}" }
      end

      def fingerprint_assets
        Dir["{css,images,js}/**/*"].each do |source|
          next unless File.file?(source)
          fingerprint_asset(source)
        end
      end

      def render_asset(source, asset, output)
        @assets[asset] = output
        @stasis.controller.before output do
          begin
            @asset_path = asset
            instead render(source, write: false)
          ensure
            @asset_path = nil
          end
        end
      end
    end
  end
end
