require 'i18n'

class Stasis
  module Extensions
    class I18n < Stasis::Plugin
      before_all :before_all
      reset :reset

      def initialize(stasis)
        @stasis = stasis
        reset
      end

      def before_all
        ::I18n.locale = ENV['LOCALE'] || ::I18n.default_locale
        define_helpers
      end

      def reset
        ::I18n.load_path = Dir[File.expand_path('config/locales/*.yml', @stasis.root)]
        ::I18n.default_locale = :en
        ::I18n.reload!
      end

      protected

      def define_helpers
        @stasis.controller.helpers do
          def body_class
            [::I18n.locale, scope.last]
          end

          def relative_path
            @asset_path || _stasis.path.sub("#{_stasis.root}/", '')
          end

          def root_url
            "http://#{t('hostname', :scope => '')}"
          end

          def scope
            relative_path.sub(/\.\w+(\.(?:erb|haml|scss))?$/, '').split('/')
          end

          def translate(*args)
            options = args.last.is_a?(Hash) ? args.pop : {}
            options[:scope] ||= scope
            ::I18n.translate(*args, options)
          end

          alias_method :t, :translate
        end
      end
    end
  end
end
